unit JDWndPrc;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons;

const
   ActiveWndList : TList = nil;
   HELPINFO : THELPINFO = (                { Structure pointed to by lParam of WM_HELP }
      cbSize : sizeof(THELPINFO);          { Size in bytes of this struct  }
      iContextType: HELPINFO_MENUITEM;     { Either HELPINFO_WINDOW or HELPINFO_MENUITEM }
      iCtrlId: 0;                          { Control Id or a Menu item Id. }
      hItemHandle: 0;                      { hWnd of control or hMenu.     }
      dwContextId: 0;                      { Context Id associated with this item }
      MousePos: (x: 0; y: 0);              { Mouse Position in screen co-ordinates }
   );

type
   TWndProcFunc = function (Window: HWND; Message, WParam: Longint; LParam: Longint): Longint; stdcall;
   TProcFuncRec = record
      Wnd : HWND;
      IsContenxtHelp : boolean;
      IsMenu         : boolean;
      FCount : integer;
      case byte of
        1 : ( WndProcFunc : TWndProcFunc);
        2 : ( WndProcLong : longint)
   end;
   PProcFuncRec = ^TProcFuncRec;

function  GetHandleRecordNum( Handle : HWND ) : integer;
function  GetHandleRecord( Handle : HWND ) : PProcFuncRec;
function  AddHandleRecord( Handle : HWND; WndProc : longint) : PProcFuncRec;
procedure DelHandleRecord( Handle : HWND );
procedure DelHandleRecordNum( i : integer );
procedure DelWndList;
function  StdWndProc(Window: HWND; Message, WParam: Longint;
  LParam: Longint): Longint; stdcall;

implementation

const
   MenuHandle : integer = 0;
   IsDlg : boolean = false;
   IsProcessing : boolean = false;

function GetHandleRecordNum( Handle : HWND ) : integer;
var
   i : integer;

begin
   result := -1;
   if ActiveWndList = nil then
      exit;

   for i := 0 to ActiveWndList.Count - 1 do
      if PProcFuncRec(ActiveWndList[i])^.Wnd = Handle then begin
         result := i;
         exit;
      end;
end;

function GetHandleRecord( Handle : HWND ) : PProcFuncRec;
var
   i : integer;

begin
   result := nil;

   i := GetHandleRecordNum( Handle);
   if i >= 0 then
      result := ActiveWndList[i];
end;

function AddHandleRecord( Handle : HWND; WndProc : longint) : PProcFuncRec;
begin
   getmem(result, sizeof(TProcFuncRec));
   FillChar(result^, sizeof(TProcFuncRec), 0);

   result^.Wnd := Handle;
   result^.WndProcLong := WndProc;
   ActiveWndList.Add(result);

end;

procedure DelHandleRecord( Handle : HWND );
var
   i : integer;

begin
   i := GetHandleRecordNum( Handle);
   if i >= 0 then begin
      freemem(ActiveWndList[i], sizeof(TProcFuncRec));
      ActiveWndList.Delete(i);
   end;
end;

procedure DelHandleRecordNum( i : integer );
begin
   freemem(ActiveWndList[i], sizeof(TProcFuncRec));
   ActiveWndList.Delete(i);
end;

procedure DelWndList;
var
   i : integer;
begin
   for i := ActiveWndList.Count - 1 downto 0 do
      DelHandleRecordNum(i);

   ActiveWndList.Free;
end;

function StdWndProc(Window: HWND; Message, WParam: Longint;
  LParam: Longint): Longint; stdcall;
var
   ProcFuncRec : PProcFuncRec;

begin
   ProcFuncRec := GetHandleRecord(Window);

   case Message of
     WM_SYSCOMMAND :
        if (WParam = SC_CONTEXTHELP) then begin
           ProcFuncRec^.IsContenxtHelp := true;
           MenuHandle := 0;
        end;
     WM_HELP : begin
        ProcFuncRec^.IsContenxtHelp := false;
        MenuHandle := 0;
     end;
     WM_MENUSELECT :
        if ProcFuncRec^.IsMenu then
           if DWORD(WParam) <> $ffff0000 then
              if ProcFuncRec^.IsContenxtHelp then
                 if (WParam and (MF_POPUP shl 16)) = (MF_POPUP shl 16) then
                    SetCursor( Screen.Cursors[crHelp])
                 else
                    MenuHandle := LParam;
     WM_COMMAND :
        if ProcFuncRec^.IsMenu then
           if (WParam and $ffff0000) = 0 then
              if ProcFuncRec^.IsContenxtHelp then begin
                 HELPINFO.iContextType := HELPINFO_MENUITEM;
                 HELPINFO.iCtrlId := WParam and $0000ffff;
                 HELPINFO.hItemHandle := MenuHandle;
                 HELPINFO.dwContextId := 0;
                 ProcFuncRec^.WndProcFunc(Window, WM_HELP, 0, longint(@HELPINFO));
                 ProcFuncRec^.IsContenxtHelp := false;
                 MenuHandle := 0;
                 result := 0;
                 exit;
              end;
   end;

   result := ProcFuncRec^.WndProcFunc(Window, Message, WParam, LParam);
end;

initialization
   ActiveWndList := TList.Create;
finalization
   DelWndList;
end.




